import contextlib
import mysql.connector
import redis

from flask import current_app, g


def get_redis():  # pragma: no cover
    if 'redis_db' not in g:
        config = current_app.config['INVENTORY_PROVIDER_CONFIG']
        g.redis_db = redis.Redis(
            host=config['redis']['hostname'],
            port=config['redis']['port'])

    return g.redis_db


@contextlib.contextmanager
def connection(db_params):
    cx = None
    try:
        cx = mysql.connector.connect(
            host=db_params["hostname"],
            user=db_params["username"],
            passwd=db_params["password"],
            db=db_params["dbname"])
        yield cx
    finally:
        if cx:
            cx.close()


@contextlib.contextmanager
def cursor(cnx):  # pragma: no cover
    csr = None
    try:
        csr = cnx.cursor()
        yield csr
    finally:
        if csr:
            csr.close()
